@@ -26,14 +26,25 @@
 #include <gpgme.h>
 
 #include "seahorse-context.h"
+#include "seahorse-widget.h"
 
-const gchar*	seahorse_passphrase_get	(SeahorseContext	*sctx,
-					 const gchar		*desc,
-					 gpointer		*data);
+#define SEAHORSE_PASS_BAD    0x00000001
+#define SEAHORSE_PASS_NEW    0x01000000
 
-GpgmeRecipients	seahorse_recipients_get	(SeahorseContext	*sctx);
+gpgme_error_t
+seahorse_passphrase_get (SeahorseContext *sctx, const gchar *passphrase_hint, 
+                            const char* passphrase_info, int prev_bad, int fd);
 
-void		seahorse_signatures_new	(SeahorseContext	*sctx,
-					 GpgmeSigStat		status);
+GList*      	seahorse_recipients_get	(SeahorseContext	*sctx);
+
+SeahorseWidget*	seahorse_signatures_new	(SeahorseContext	*sctx);
+
+void            seahorse_signatures_add (SeahorseContext *sctx, 
+                                            SeahorseWidget *widget,
+                                            const gchar* data, 
+                                            gpgme_verify_result_t status);
+                            
+void            seahorse_signatures_run (SeahorseContext *sctx, 
+                                            SeahorseWidget *swidget);
 
 #endif /* __SEAHORSE_LIBDIALOGS_H__ */
