@@ -23,6 +23,7 @@
 #include <gnome.h>
 #include <gconf/gconf-client.h>
 #include <eel/eel-gconf-extensions.h>
+#include <libgnomevfs/gnome-vfs.h>
 
 #include "seahorse-windows.h"
 #include "seahorse-widget.h"
@@ -32,25 +33,32 @@
 #include "seahorse-key-manager-store.h"
 #include "seahorse-key-dialogs.h"
 #include "seahorse-key-op.h"
+#include "seahorse-key-widget.h"
+#include "seahorse-op.h"
 
 #define KEY_LIST "key_list"
 
-#define STATUSBAR_VISIBLE UI_SCHEMAS "/statusbar_visible"
-#define TOOLBAR_VISIBLE UI_SCHEMAS "/toolbar_visible"
-
-#define GNOME_INTERFACE "/desktop/gnome/interface"
-#define GNOME_TOOLBAR_STYLE GNOME_INTERFACE "/toolbar_style"
-
 static guint signal_id = 0;
 static gulong hook_id = 0;
 
+/* Drag and trop target types */
+enum TargetTypes {
+    TEXT_PLAIN,
+    TEXT_URIS
+};
+
+/* Drag and drop targent entries */
+static const GtkTargetEntry target_entries[] = {
+   { "text/uri-list", 0, TEXT_URIS },
+   { "text/plain", 0, TEXT_PLAIN }
+};
+
 /* Quits seahorse */
 static void
 quit (GtkWidget *widget, SeahorseWidget *swidget)
 {
 	g_signal_remove_emission_hook (signal_id, hook_id);
 	seahorse_context_destroy (swidget->sctx);
-	gtk_exit (0);
 }
 
 /* Quits seahorse */
@@ -65,14 +73,132 @@ delete_event (GtkWidget *widget, GdkEvent *event, SeahorseWidget *swidget)
 static void
 generate_activate (GtkWidget *widget, SeahorseWidget *swidget)
 {
-	seahorse_generate_show (swidget->sctx);
+	seahorse_generate_select_show (swidget->sctx);
+}
+/* Loads Key generation assistant for first time users */
+static void
+new_button_clicked (GtkWidget *widget, SeahorseWidget *swidget)
+{
+	seahorse_generate_druid_show (swidget->sctx);
+}
+
+/* Setup our file types on a file chooser dialog */
+static void
+setup_file_types (GtkFileChooser* dialog)
+{
+    GtkFileFilter* filter = gtk_file_filter_new ();
+    gtk_file_filter_set_name (filter, _("All PGP key files"));
+    gtk_file_filter_add_pattern (filter, "*.asc");    
+    gtk_file_filter_add_pattern (filter, "*.key");    
+    gtk_file_filter_add_pattern (filter, "*.pkr");    
+    gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);    
+    gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (dialog), filter);
+
+    filter = gtk_file_filter_new ();
+    gtk_file_filter_set_name (filter, _("All files"));
+    gtk_file_filter_add_pattern (filter, "*");    
+    gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);    
 }
 
 /* Loads import dialog */
 static void
 import_activate (GtkWidget *widget, SeahorseWidget *swidget)
 {
-	seahorse_import_show (swidget->sctx);
+    SeahorseKeySource *sksrc;
+    GtkWidget *dialog;
+    char* uri = NULL;
+    gint keys;
+    gpgme_error_t err;
+    
+    sksrc = seahorse_context_get_pri_source (swidget->sctx);
+    g_return_if_fail (sksrc != NULL);
+    
+    dialog = gtk_file_chooser_dialog_new (_("Import Key"), 
+                GTK_WINDOW(glade_xml_get_widget (swidget->xml, "key-manager")),
+                GTK_FILE_CHOOSER_ACTION_OPEN, 
+                GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+                NULL);
+
+    setup_file_types (GTK_FILE_CHOOSER (dialog));
+    gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (dialog), FALSE);
+     
+    if(gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
+        uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
+        
+    gtk_widget_destroy (dialog);
+
+    if(uri) {
+        keys = seahorse_op_import_file (sksrc, uri, &err);
+        
+        if (GPG_IS_OK (err))
+            seahorse_key_source_refresh (sksrc, FALSE);
+        else
+            seahorse_util_handle_error (err, _("Couldn't import keys from \"%s\""), 
+                seahorse_util_uri_get_last (uri));
+                
+        g_free (uri);
+    }
+}
+
+/* Callback for pasting from clipboard */
+static void
+clipboard_received (GtkClipboard *board, const gchar *text, SeahorseContext *sctx)
+{
+    SeahorseKeySource *sksrc;
+    gpgme_error_t err;
+    gint keys;
+    
+    sksrc = seahorse_context_get_pri_source (sctx);
+    g_return_if_fail (sksrc != NULL);
+ 
+    keys = seahorse_op_import_text (sksrc, text, &err);
+ 
+    if (!GPG_IS_OK (err))
+        seahorse_util_handle_error (err, _("Couldn't import keys from clipboard"));
+    else if (keys > 0)
+        seahorse_key_source_refresh (sksrc, FALSE);
+}
+
+/* Pastes key from keyboard */
+static void 
+paste_activate (GtkWidget *widget, SeahorseWidget *swidget)
+{
+    GdkAtom atom;
+    GtkClipboard *board;
+       
+    atom = gdk_atom_intern ("CLIPBOARD", FALSE);
+    board = gtk_clipboard_get (atom);
+    gtk_clipboard_request_text (board,
+         (GtkClipboardTextReceivedFunc)clipboard_received, swidget->sctx);
+}
+
+/* Copies key to clipboard */
+static void
+copy_activate (GtkWidget *widget, SeahorseWidget *swidget)
+{
+    GdkAtom atom;
+    GtkClipboard *board;
+    gchar *text;
+    gpgme_error_t err;
+    GList *keys;
+  
+    keys = seahorse_key_store_get_selected_keys (GTK_TREE_VIEW (
+                glade_xml_get_widget (swidget->xml, KEY_LIST)));
+       
+    if (g_list_length (keys) == 0)
+        return;
+               
+    text = seahorse_op_export_text (keys, &err);
+
+    if (!GPG_IS_OK (err))
+        seahorse_util_handle_error (err, _("Couldn't export key to clipboard"));
+    else if (text != NULL) {
+        atom = gdk_atom_intern ("CLIPBOARD", FALSE);
+        board = gtk_clipboard_get (atom);
+        gtk_clipboard_set_text (board, text, strlen (text));
+        g_free (text);
+    }
 }
 
 /* Loads key properties if a key is selected */
@@ -91,11 +217,68 @@ properties_activate (GtkWidget *widget, SeahorseWidget *swidget)
 static void
 export_activate (GtkWidget *widget, SeahorseWidget *swidget)
 {
-	GpgmeRecipients recips;
-	
-	recips = seahorse_key_store_get_selected_recips (GTK_TREE_VIEW (
-		glade_xml_get_widget (swidget->xml, KEY_LIST)));
-	seahorse_export_show (swidget->sctx, recips);
+    GtkWidget *dialog;
+    char* uri = NULL;
+    gpgme_error_t err;
+    GList *keys;
+    
+    dialog = gtk_file_chooser_dialog_new (_("Export Key"), 
+                GTK_WINDOW(glade_xml_get_widget (swidget->xml, "key-manager")),
+                GTK_FILE_CHOOSER_ACTION_SAVE, 
+                GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+                NULL);
+
+    setup_file_types (GTK_FILE_CHOOSER(dialog));
+    gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (dialog), FALSE);
+     
+    while(gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
+        GnomeVFSURI *u = gnome_vfs_uri_new (gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(dialog)));
+        if (u != NULL) {
+            if (gnome_vfs_uri_exists(u)) {
+                GtkWidget* edlg;
+                edlg = gtk_message_dialog_new_with_markup (GTK_WINDOW (dialog),
+                    GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_QUESTION,
+                    GTK_BUTTONS_NONE, _("<b>A file already exists with this name.</b>\n\nDo you want to replace it with the one you are saving?"));
+                gtk_dialog_add_buttons (GTK_DIALOG (edlg), 
+                       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                       _("Replace"), GTK_RESPONSE_ACCEPT, NULL);
+
+                gtk_dialog_set_default_response (GTK_DIALOG (edlg), GTK_RESPONSE_CANCEL);                    
+                
+                if (gtk_dialog_run (GTK_DIALOG (edlg)) == GTK_RESPONSE_ACCEPT)
+                    uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
+
+                gtk_widget_destroy (edlg);
+
+            /* File doesn't exist */
+            } else {
+                uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
+            }
+            
+            gnome_vfs_uri_unref (u);
+
+        }
+
+        if (uri != NULL)
+            break;
+    }
+
+    gtk_widget_destroy (dialog);
+
+    if(uri) {
+        keys = seahorse_key_store_get_selected_keys (GTK_TREE_VIEW (
+            glade_xml_get_widget (swidget->xml, KEY_LIST)));
+
+        seahorse_op_export_file (keys, uri, &err); 
+		g_list_free (keys);
+		        
+        if (!GPG_IS_OK (err))
+            seahorse_util_handle_error (err, _("Couldn't export key to \"%s\""),
+                    seahorse_util_uri_get_last (uri));
+                    
+        g_free (uri);
+    }
 }
 
 static void
@@ -127,7 +310,7 @@ change_passphrase_activate (GtkMenuItem *item, SeahorseWidget *swidget)
 	skey = seahorse_key_store_get_selected_key (GTK_TREE_VIEW (
 		glade_xml_get_widget (swidget->xml, KEY_LIST)));
 	if (skey != NULL && SEAHORSE_IS_KEY_PAIR (skey))
-		seahorse_key_pair_op_change_pass (swidget->sctx, SEAHORSE_KEY_PAIR (skey));
+		seahorse_key_pair_op_change_pass (SEAHORSE_KEY_PAIR (skey));
 }
 
 static void
@@ -170,12 +353,6 @@ preferences_activate (GtkWidget *widget, SeahorseWidget *swidget)
 	seahorse_preferences_show (swidget->sctx);
 }
 
-static void
-view_bar (GtkCheckMenuItem *item, const gchar *key)
-{
-	eel_gconf_set_boolean (key, gtk_check_menu_item_get_active (item));
-}
-
 static void
 expand_all_activate (GtkMenuItem *item, SeahorseWidget *swidget)
 {
@@ -199,6 +376,7 @@ about_activate (GtkWidget *widget, SeahorseWidget *swidget)
 		"Jacob Perkins <jap1@users.sourceforge.net>",
 		"Jose Carlos Garcia Sogo <jsogo@users.sourceforge.net>",
 		"Jean Schurger <yshark@schurger.org>",
+        "Nate Nielsen <nielsen@memberwebs.com>",
 		NULL
 	};
 
@@ -271,6 +449,7 @@ set_key_options_sensitive (SeahorseWidget *swidget, gboolean selected, gboolean
 	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "delete"), selected);
 	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "export_button"), selected);
 	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "export"), selected);
+    gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "copy_key"), selected);
 	
 	/* items that can do single */
 	create = (skey != NULL && seahorse_key_widget_can_create ("key-properties", skey));
@@ -298,7 +477,6 @@ selection_changed (GtkTreeSelection *selection, SeahorseWidget *swidget)
 	gint rows = 0;
 	gboolean selected = FALSE, secret = FALSE;
 	SeahorseKey *skey = NULL;
-	GtkTreeView *view;
 	
 	rows = gtk_tree_selection_count_selected_rows (selection);
 	selected = rows > 0;
@@ -346,65 +524,27 @@ key_list_popup_menu (GtkWidget *widget, SeahorseWidget *swidget)
 	skey = seahorse_key_store_get_selected_key (GTK_TREE_VIEW (widget));
 	if (skey != NULL)
 		show_context_menu (swidget, 0, gtk_get_current_event_time ());
+       
+    return FALSE;
 }
 
-static void
-set_toolbar_style (GtkToolbar *toolbar, const gchar *style)
-{
-	if (g_str_equal (style, TOOLBAR_BOTH))
-		gtk_toolbar_set_style (toolbar, GTK_TOOLBAR_BOTH);
-	else if (g_str_equal (style, TOOLBAR_BOTH_HORIZ))
-		gtk_toolbar_set_style (toolbar, GTK_TOOLBAR_BOTH_HORIZ);
-	else if (g_str_equal (style, TOOLBAR_TEXT))
-		gtk_toolbar_set_style (toolbar, GTK_TOOLBAR_TEXT);
-	else if (g_str_equal (style, TOOLBAR_ICONS))
-		gtk_toolbar_set_style (toolbar, GTK_TOOLBAR_ICONS);
-}
 
 static void
 gconf_notification (GConfClient *gclient, guint id, GConfEntry *entry, SeahorseWidget *swidget)
 {
 	const gchar *key;
 	GConfValue *value;
-	GtkWidget *widget;
 	
 	key = gconf_entry_get_key (entry);
 	value = gconf_entry_get_value (entry);
-	
-	if (g_str_equal (key, STATUSBAR_VISIBLE)) {
-		widget = glade_xml_get_widget (swidget->xml, "status");
-		
-		if (gconf_value_get_bool (value))
-			gtk_widget_show (widget);
-		else
-			gtk_widget_hide (widget);
-	}
-	else if (g_str_equal (key, TOOLBAR_VISIBLE)) {
-		widget = glade_xml_get_widget (swidget->xml, "tool_dock");
-		
-		if (gconf_value_get_bool (value))
-			gtk_widget_show (widget);
-		else
-			gtk_widget_hide (widget);
-	}
-	else if (g_str_equal (key, GNOME_TOOLBAR_STYLE) &&
-	g_str_equal (TOOLBAR_DEFAULT, eel_gconf_get_string (TOOLBAR_STYLE_KEY))) {
-		set_toolbar_style (GTK_TOOLBAR (glade_xml_get_widget (swidget->xml, "toolbar")),
-			gconf_value_get_string (value));
-	}
-	else if (g_str_equal (key, TOOLBAR_STYLE_KEY)) {
-		widget = glade_xml_get_widget (swidget->xml, "toolbar");
-		
-		/* if changed to default, use system settings */
-		if (g_str_equal (gconf_value_get_string (value), TOOLBAR_DEFAULT)) {
-			set_toolbar_style (GTK_TOOLBAR (widget),
-				eel_gconf_get_string (GNOME_TOOLBAR_STYLE));
-		}
-		else
-			set_toolbar_style (GTK_TOOLBAR (widget), gconf_value_get_string (value));
-	}
+
+    /* 
+     * Removed last two gconf notification items, although I'm sure
+     * we'll get more, so leaving this code here... 
+     */
 }
-/*
+
+#if 0
 static void
 show_progress (SeahorseContext *sctx, const gchar *op, gdouble fract, SeahorseWidget *swidget)
 {
@@ -417,14 +557,14 @@ show_progress (SeahorseContext *sctx, const gchar *op, gdouble fract, SeahorseWi
 	status = GNOME_APPBAR (glade_xml_get_widget (swidget->xml, "status"));
 	gnome_appbar_set_status (status, op);
 	progress = gnome_appbar_get_progress (status);
-	/* do progress *
+	/* do progress */
 	if (fract <= 1 && fract > 0)
 		gtk_progress_bar_set_fraction (progress, fract);
 	else if (fract != -1) {
 		gtk_progress_bar_set_pulse_step (progress, 0.05);
 		gtk_progress_bar_pulse (progress);
 	}
-	/* if fract == -1, cleanup progress *
+	/* if fract == -1, cleanup progress */
 	else
 		gtk_progress_bar_set_fraction (progress, 0);
 	
@@ -439,18 +579,19 @@ show_progress (SeahorseContext *sctx, const gchar *op, gdouble fract, SeahorseWi
 	while (g_main_context_pending (NULL))
 		g_main_context_iteration (NULL, TRUE);
 }
-*/
+#endif
+
 /* params[0] = sctx, params[1] = op string, params[2] = fract */
 static gboolean
 progress_hook (GSignalInvocationHint *hint, guint n_params, const GValue *params, SeahorseWidget *swidget)
 {
 	GnomeAppBar *status;
 	GtkProgressBar *progress;
-	//gboolean sensitive;
+	/* gboolean sensitive; */
 	gdouble fract;
 
 	fract = g_value_get_double (&params[2]);
-	//sensitive = (fract == -1);
+	/* sensitive = (fract == -1); */
 	
 	status = GNOME_APPBAR (glade_xml_get_widget (swidget->xml, "status"));
 	gnome_appbar_set_status (status, g_value_get_string (&params[1]));
@@ -472,14 +613,67 @@ progress_hook (GSignalInvocationHint *hint, guint n_params, const GValue *params
 	return TRUE;
 }
 
-void
+static void
+target_drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gint y,
+                           GtkSelectionData *data, guint info, guint time, SeahorseContext *sctx)
+{
+    SeahorseKeySource *sksrc;
+    gint keys = 0;
+    gpgme_error_t err;
+    gchar** uris;
+    gchar** u;
+    
+    g_return_if_fail (data != NULL);
+    
+    sksrc = seahorse_context_get_pri_source (sctx);
+    g_return_if_fail (sksrc != NULL);
+    
+    switch(info) {
+    case TEXT_PLAIN:
+        keys = seahorse_op_import_text (sksrc, data->data, &err);
+        break;
+        
+    case TEXT_URIS:
+        uris = g_strsplit (data->data, "\n", 0);
+        for(u = uris; *u; u++) {
+            g_strstrip (*u);
+            if ((*u)[0]) { /* Make sure it's not an empty line */
+                keys += seahorse_op_import_file (sksrc, *u, &err);  
+                if (!GPG_IS_OK (err)) {
+                    seahorse_util_handle_error (err, _("Couldn't import key from \"%s\""),
+                            seahorse_util_uri_get_last (*u));
+                    break;
+                }
+            }
+        }
+        g_strfreev (uris);
+        break;
+        
+    default:
+        g_assert_not_reached ();
+        return;
+    } 
+
+    if (keys > 0)
+        seahorse_key_source_refresh (sksrc, FALSE);        
+}
+
+/* Refilter the keys when the filter text changes */
+static void
+filter_changed (GtkWidget *widget, SeahorseKeyStore* skstore)
+{
+    const gchar* text = gtk_entry_get_text (GTK_ENTRY (widget));
+    g_object_set (skstore, "filter", text, NULL);
+}
+
+GtkWindow* 
 seahorse_key_manager_show (SeahorseContext *sctx)
 {
 	SeahorseWidget *swidget;
 	GtkTreeView *view;
+    GtkWidget* w;
 	GtkTreeSelection *selection;
-	GtkWidget *widget;
-	gboolean visible;
+    SeahorseKeyStore *skstore;
 	
 	swidget = seahorse_widget_new ("key-manager", sctx);
 	gtk_object_sink (GTK_OBJECT (sctx));
@@ -530,6 +724,8 @@ seahorse_key_manager_show (SeahorseContext *sctx)
 		G_CALLBACK (sign_activate), swidget);
 	glade_xml_signal_connect_data (swidget->xml, "delete_activate",
 		G_CALLBACK (delete_activate), swidget);
+    glade_xml_signal_connect_data (swidget->xml, "copy_activate",
+        G_CALLBACK (copy_activate), swidget);      
 	/* selected key with secret signals */
 	glade_xml_signal_connect_data (swidget->xml, "change_passphrase_activate",
 		G_CALLBACK (change_passphrase_activate), swidget);
@@ -548,32 +744,18 @@ seahorse_key_manager_show (SeahorseContext *sctx)
 	/* init gclient */
 	eel_gconf_notification_add (UI_SCHEMAS, (GConfClientNotifyFunc) gconf_notification, swidget);
 	eel_gconf_monitor_add (UI_SCHEMAS);
-	eel_gconf_notification_add (GNOME_INTERFACE, (GConfClientNotifyFunc) gconf_notification, swidget);
-	eel_gconf_monitor_add (GNOME_INTERFACE);
-	
-	/* init toolbar */
-	glade_xml_signal_connect_data (swidget->xml, "toolbar_activate",
-		G_CALLBACK (view_bar), TOOLBAR_VISIBLE);
-	set_toolbar_style (GTK_TOOLBAR (glade_xml_get_widget (swidget->xml, "toolbar")),
-		eel_gconf_get_string (TOOLBAR_STYLE_KEY));
-	visible = eel_gconf_get_boolean (TOOLBAR_VISIBLE);
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (
-		glade_xml_get_widget (swidget->xml, "view_toolbar")), visible);
-	if (!visible)
-		gtk_widget_hide (glade_xml_get_widget (swidget->xml, "tool_dock"));
-	
-	/* init status bar */
-	widget = glade_xml_get_widget (swidget->xml, "view_statusbar");
-	visible = eel_gconf_get_boolean (STATUSBAR_VISIBLE);
-	glade_xml_signal_connect_data (swidget->xml, "statusbar_activate",
-		G_CALLBACK (view_bar), STATUSBAR_VISIBLE);
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (widget), visible);
-	if (!visible)
-		gtk_widget_hide (glade_xml_get_widget (swidget->xml, "status"));
 	
+	/* first time signals */
+	glade_xml_signal_connect_data (swidget->xml, "import_button_clicked",
+		G_CALLBACK (import_activate), swidget);
+	glade_xml_signal_connect_data (swidget->xml, "new_button_clicked",
+		G_CALLBACK (new_button_clicked), swidget);
+		
 	/* other signals */	
 	glade_xml_signal_connect_data (swidget->xml, "preferences_activate",
 		G_CALLBACK (preferences_activate), swidget);
+    glade_xml_signal_connect_data (swidget->xml, "paste_activate",
+       G_CALLBACK (paste_activate), swidget);
 	glade_xml_signal_connect_data (swidget->xml, "about_activate",
 		G_CALLBACK (about_activate), swidget);
 	
@@ -583,6 +765,27 @@ seahorse_key_manager_show (SeahorseContext *sctx)
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
 	g_signal_connect (selection, "changed",
 		G_CALLBACK (selection_changed), swidget);
-	seahorse_key_manager_store_new (sctx, view);
+    skstore = seahorse_key_manager_store_new (sctx, view);
 	selection_changed (selection, swidget);
+   
+    w = glade_xml_get_widget (swidget->xml, "key-manager");
+    
+    /* Setup drops */
+    gtk_drag_dest_set (w, GTK_DEST_DEFAULT_ALL, target_entries, 
+            sizeof (target_entries) / sizeof (target_entries[0]), GDK_ACTION_COPY);
+    gtk_signal_connect (GTK_OBJECT (w), "drag_data_received",
+            GTK_SIGNAL_FUNC (target_drag_data_received), sctx);
+                        
+    /* For the filtering */
+    glade_xml_signal_connect_data(swidget->xml, "on_filter_changed",
+                              G_CALLBACK(filter_changed), skstore);
+
+    /* Although not all the keys have completed we'll know whether we have 
+     * any or not at this point */
+	if (seahorse_context_get_n_keys (sctx) == 0) {
+		w = glade_xml_get_widget (swidget->xml, "first-time-box");
+		gtk_widget_show (w);
+	}
+	
+    return GTK_WINDOW (w);
 }
